@@ -6,7 +6,7 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-BEGIN { $| = 1; print "1..4\n"; }
+BEGIN { $| = 1; print "1..7\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use PDF::Template;
 $loaded = 1;
@@ -25,7 +25,9 @@ my @lnames = qw ( Washington Lincoln Bush Hoover Adams Kennedy Eisenhower Garfie
 
 ### Test 2
 
-my $rpt = new PDF::Template(FILENAME=>'examples/t2.xml');
+my $rpt = new PDF::Template(FILENAME=>'examples/t2.xml',
+                            INFO=>{Creator=>'Dave',Author=>'Tupac Shakur',Title=>"Test T2", Subject=>'Second Subject', Keywords=>'test two keywords pdf template'}
+                           );
 
 
 $rpt->param( { TITLE=>'blah blah',
@@ -42,17 +44,24 @@ print "ok 2\n";
 ### Test 3 : Loop
 
 my @d3;
-for (my $i=0; $i<100; $i++)
+my @d3b;
+for (my $i=0; $i<20; $i++)
 {
    my %h;
+   my %h2;
    $h{LNAME} = $lnames[rand(10)];
    $h{FNAME} = $fnames[rand(10)];
    $h{VAL} = $i;
+   
+   %h2 = %h;
+   
    push @d3,\%h;
+   push @d3b,\%h2;
 }
 
 my $t3 = new PDF::Template(FILENAME=>'examples/t3_loop.xml');
 $t3->param(DATA=>\@d3);
+$t3->param(DATA2=>\@d3b);
 
 $t3->write_file("examples/t3_loop.pdf");
 
@@ -68,3 +77,31 @@ my $t4 = new PDF::Template(FILENAME=>'examples/t4_elements.xml');
 $t4->write_file("examples/t4_elements.pdf");
 
 print "ok 4\n";
+
+### Test 5 : Images
+
+
+my $t5 = new PDF::Template(FILENAME=>'examples/t5_images.xml');
+
+$t5->write_file("examples/t5.pdf");
+
+print "ok 5\n";
+
+### Test 6 : Conditionals
+
+
+my $t6 = new PDF::Template(FILENAME=>'examples/t6_conditionals.xml');
+
+my $t6ref = [ { N=>1 }, { N=>2 }, { N=>3 }, { N=>4 }, { N=>5 } ];
+$t6->param(DATA=>$t6ref);
+$t6->write_file("examples/t6.pdf");
+
+print "ok 6\n";
+
+
+#### Test 7: Fonts
+my $t7 = new PDF::Template(FILENAME=>'examples/t7_fonts.xml');
+
+$t7->write_file("examples/t7.pdf");
+
+print "ok 7\n";
